GameGrid = class(function(self,_sizeX,_sizeY,_parent)
	self.tiles = {}
	for x=1,_sizeX do
		self.tiles[x] = {}
		for y=1,_sizeY do
			self.tiles[x][y] = false
		end
	end
	self.pos = {x=0,y=0} --this is for blocks class to understand their location when they are attached to game grid.
	self.size = {x=_sizeX,y=_sizeY}
	self.tilesize = G_WH/30
	self.currentTile = false
	--self.nextTile = GameTile(math.random(1,7),HSV(0,255,math.random(100,255)))
	self.nextTile = GameTile(math.random(1,7),HSV(math.random(0,255),255,255))
	self.blocks = {}
	self.owner = _parent
	return self
end)

GameGrid.Render = function(self,gx,gy)
	love.graphics.setLineWidth( self.tilesize/10 )
		love.graphics.rectangle( "fill", self.tilesize+gx, gy, self.tilesize*10, self.tilesize*21)
	for x=1,self.size.x do
		for y=1,self.size.y do
			if self.tiles[x] and self.tiles[x][y] then
				self.tiles[x][y]:Render(gx,gy,self.tilesize)
			end
		end
	end
	if self.currentTile then
		local t = self.currentTile:GetBlocks()
		for k,v in pairs(t) do
			v:Render(gx,gy,self.tilesize)
		end
	end
	love.graphics.setColor(255,0,0,150)
	love.graphics.line( self.tilesize+gx, self.tilesize*2+gy, self.tilesize*11+gx, self.tilesize*2+gy )
	love.graphics.setColor(255,255,255,255)
	love.graphics.rectangle( "line", self.tilesize+gx, gy, self.tilesize*10, self.tilesize*21 )
end

GameGrid.RenderNextTile = function(self,gx,gy)
	if self.nextTile then
		local t = self.nextTile:GetBlocks()
		for k,v in pairs(t) do
			v:Render(gx,gy,self.tilesize)
		end
	end
	love.graphics.setColor(255,255,255,255)
	love.graphics.rectangle( "line", gx+self.tilesize*4-5, gy-5, self.tilesize*4+10, self.tilesize*2+10 )
end

GameGrid.GetCurrentTile = function(self)
	if self.currentTile == false then
		return nil
	else
		return self.currentTile
	end
end

GameGrid.GetPos = function(self)
	return self.pos.x, self.pos.y
end

GameGrid.UpdateGridDown = function(self)
	if self.currentTile then
		if self:GetCurrentTileCollision(0,1) then
				self:RenderTileToGrid()
				self:ForceNewTile()
		else
			self.currentTile:Move(0,1)
		end
	else
		self:ForceNewTile()
	end
end

GameGrid.GridMove = function(self,x,y)
	if self.currentTile then
		if self:GetCurrentTileCollision(x,y) then
			if y ~= 0 then
				self:RenderTileToGrid()
				self:ForceNewTile()
				return false
			end
		else
			self.currentTile:Move(x,y)
		end
	else
		self:ForceNewTile()
		return false
	end
	return true
end

GameGrid.GridRotate = function(self,_d)
	local d = _d or 1
	if self.currentTile then
		if self:GetCurrentTileRotCollision(d) then
		else
			self.currentTile:Rotate(d)
		end
	else
		self:ForceNewTile()
	end
end

--collision!!
GameGrid.GetGridCollision = function(self,x,y,b)
	if self.tiles[x] == nil then
		return true --outside bounds horizontal
	elseif y < 1 then
		return false --above
	elseif self.tiles[x][y] == nil then
		return true --bellow
	else
			local dx,dy = b:GetPos(true)
			if self.tiles[x][y] == false then
				return false
			else
				print("we have problem " .. dx .. "/" .. dy .. " " .. x .. "/" .. y)
				return self.tiles[x][y]:Collision(dx,dy)
			end
	end
end

GameGrid.GetCurrentTileRotCollision = function(self,d)
	if self.currentTile then
		local t = self.currentTile:GetTileRotPos(d)
		local b = self.currentTile:GetBlocks()
		for k,v in pairs(t) do
			if self:GetGridCollision(v.x,v.y,b[k]) then
				return true
			end
		end
	end
	return false
end

GameGrid.GetCurrentTileCollision = function(self,dx,dy)
	if self.currentTile then
		local t = self.currentTile:GetBlocks()
		for k,v in pairs(t) do
			local x,y = v:GetPos(true)
			if self:GetGridCollision(dx+x,dy+y,v) then
				return true
			end
		end
	end
	return false
end


-------stUFF

GameGrid.IsCurrentTileAt = function(self,x,y)
	if self.currentTile then
		local t = self.currentTile:GetTileShapePos()
		for k,v in pairs(t) do
			if x == v.x and y == v.y then
				return true
			end
		end
	end
	return false
end

GameGrid.RenderTileToGrid = function(self)
		local gameEnd = false
		if self.currentTile then
			local t = self.currentTile:GetBlocks()
			for k,v in pairs(t) do
				local x,y = v:GetPos(true)
				if self.tiles[x] ~= nil and self.tiles[x][y] ~= nil then
					self.tiles[x][y] = v
					v:SetParent(self,true)
				end
				if y == 1 then gameEnd = true end
			end
		end
		if gameEnd == true then
			self:GameEnd()
		end
		self:CheckLines()
end

GameGrid.GameEnd = function(self)
	self.currentTile:Destroy()
	self.owner:GameEnd()
end

GameGrid.GameStart = function(self)
		self.tiles = {}
		for x=1,self.size.x do
			self.tiles[x] = {}
			for y=1,self.size.y do
				self.tiles[x][y] = false
			end
		end
	self.currentTile = false
	self.nextTile = GameTile(math.random(1,7),HSV(math.random(0,255),255,255))
	--self.nextTile = GameTile(math.random(1,7),HSV(0,255,math.random(50,255)))
end

GameGrid.ForceNewTile = function(self,t)
		if self.currentTile then
			self.currentTile:Destroy()
		end
		if not G_GameEnded then
		self.currentTile = self.nextTile
		--self.nextTile = GameTile(math.random(1,7),HSV(0,255,math.random(50,255)))
		self.nextTile = GameTile(math.random(1,7),HSV(math.random(0,255),255,255))
		else
		self.currentTile = nil
		if self.nextTile then
			self.nextTile:Destroy()
		end
		self.nextTile = nil
		end
end

GameGrid.CheckLines = function(self)
	local t = {}
	for y=1,self.size.y do
		local n = 0
		for x=1,self.size.x do
			if self.tiles[x][y] then n = n +1 end
		end
		if n == self.size.x then
			table.insert(t,y)
		end
	end
	local scored = false
		for k,i in pairs(t) do
			--DDPrint("scored line: " .. i)
			self.owner:Score(k)
			scored = true
			for j=0,i-1 do
				local y = i-j
				for x=1,self.size.x do
					if self.tiles[x][y-1] ~= nil then
						self.tiles[x][y] = self.tiles[x][y-1]
						if self.tiles[x][y] ~= false then
							self.tiles[x][y]:SetPos(x,y)
						end
					else
						if self.tiles[x][y] ~= false then
							self.tiles[x][y]:Destroy()
						end
						self.tiles[x][y] = false
					end
				end
			end
		end
		if scored then G_SoundScore:play() end
end
