GamePlayer = class(function(self,id,_js)
	--id is used to identify controller used.
	print("player with id #" .. id .. " added")
	self.game_grid = GameGrid(10,20,self)
	self.id = id
	self.lastInput = {true,true}
	self.active = true
	self.visible = true
	self.paused = true
	self.score = 0
	self.joystick = _js
	--game timing
	self.GameTimer = 0
	self.GameSpeed = 1
	self.PostEffect = 16
	self.VoteQuit = false
	return self
end)

GamePlayer.GetId = function (self)
	return self.id
end

GamePlayer.IsActive = function (self)
	return self.visible
end

GamePlayer.Disconnected = function(self)
	self.paused = true
	self.visible = false
end

GamePlayer.Reconnected = function(self,_js)
	self.joystick = _js
	self.visible = true
end

GamePlayer.ApplyJInput = function (self,axis,value)
	if self.paused or not self.active then return end
	local ok = self:InputUpdate(axis,value)
	if ok then
		if axis == 1 then
			if value > G_Sensitivity.h then
					self.game_grid:GridMove(1,0)
			elseif value < G_Sensitivity.h*-1 then
					self.game_grid:GridMove(-1,0)
			end
		elseif axis == 2 then
			if value > G_Sensitivity.d then
					self:FlushDown()
			elseif value < G_Sensitivity.u*-1 then
					self.game_grid:GridRotate()
			end
		end
	end
	if axis > 2 then
			print( self.id .. " used axis " .. axis)
	end
end

GamePlayer.ApplyBInput = function (self,button,js)
	print( self.id .. " used button " .. button)
	if js ~= nil then
		if js:isDown(10) and js:isDown(9) and js:isDown(8) and js:isDown(7) then
			if self.VoteQuit then
				G_QuitVote = G_QuitVote - 1
			else
				G_QuitVote = G_QuitVote + 1
			end
			self.VoteQuit = (self.VoteQuit == false)
		end
	end
	if button == 3 then
		if self.active then
			self:Pause()
		else
			self:GameStart()
		end
	elseif button == 5 then
		ToggleMusic()
	end
	if self.paused or not self.active then return end
	if button == 9 then
			self.game_grid:GridRotate(1)
	elseif button == 10 then
			self.game_grid:GridRotate(-1)
	elseif button == 8 then
			self.game_grid:GridMove(1,0)
	elseif button == 7 then
			self.game_grid:GridMove(-1,0)
	end
end

GamePlayer.FlushDown = function (self)
	local n = 1
	while (self.game_grid:GridMove(0,1)) do
		self.score = self.score + n
		n = n+1
	end
end

GamePlayer.InputUpdate = function (self,axis,value)
	local c = self.lastInput[axis]
	self.lastInput[axis] = ( 0.5 > value and value > -0.5 )
	return c
end

GamePlayer.Destroy = function (self)
	self = nil
end

GamePlayer.RenderGrid = function (self,max,id)
	if self.visible then
			local playerSize = self.game_grid.tilesize*11
			local stepsback = (playerSize/2)*G_PlayerCount
			local mid = G_WW/2
			local location = ((id-1)*playerSize)+mid-stepsback-self.game_grid.tilesize/2
			local x = location
			local y = 50
			if not self.active then
				love.graphics.setColor(100,0,0,150)
			elseif self.paused then
				love.graphics.setColor(0,50,100,150)
			else
				love.graphics.setColor(0,0,0,150)
			end
			self.game_grid:Render(x,y)
			self.game_grid:RenderNextTile(x,y+self.game_grid.tilesize*25)
			love.graphics.setColor(255,255,255,255)
			love.graphics.print("Score: " .. self.score, self.game_grid.tilesize+x,self.game_grid.tilesize*23 )
			if not self.active then
				love.graphics.print("Game Over\n"..self.score, self.game_grid.tilesize*2+x,self.game_grid.tilesize*10 )
			elseif self.paused then
				love.graphics.print("Paused", self.game_grid.tilesize*2+x,self.game_grid.tilesize*5 )
				if self.id == 666 then
					love.graphics.print("Keypad:\n8 = Up (Rotate)\n5 = Down\n4 = Left\n6 = Right\n7 = CCW Rotate\n9 = CW Rotate\n0 = Pause", self.game_grid.tilesize*1.5+x,self.game_grid.tilesize*7 )
				elseif self.id == 667 then
					love.graphics.print("WASD:\nW = Up (Rotate)\nS = Down\nA = Left\nD = Right\nQ = CCW Rotate\nE = CW Rotate\nTab = Pause", self.game_grid.tilesize*1.5+x,self.game_grid.tilesize*7 )
				else
					love.graphics.print("Controller:\nLeft Stick\nBottom Triggers\nTop Triggers\nButton 3 = Pause\nButton 5 = Music\n\nAll Triggers:\nVote Quit", self.game_grid.tilesize*1.5+x,self.game_grid.tilesize*7 )
				end
			end
			if self.VoteQuit then
				love.graphics.setColor(100,0,0,255)
				love.graphics.print("Voted To Quit", self.game_grid.tilesize*2+x,self.game_grid.tilesize )
			end
	end
end

GamePlayer.UpdateGrid = function (self,dt)
	if not self.paused and self.active then
		self.GameTimer = self.GameTimer + dt
		if self.GameTimer >= self.GameSpeed then
			self.GameTimer = self.GameTimer - self.GameSpeed
			self.game_grid:UpdateGridDown()
		end
	end
end

GamePlayer.SetActive = function (self,bool)
	if bool ~= nil then
		self.active = bool
	else
		self.active = (self.active == false)
	end
end

GamePlayer.SetVisible = function (self,bool)
	if bool ~= nil then
		self.visible = bool
	else
		self.visible = (self.visible == false)
	end
end

GamePlayer.Score = function (self,n)
	self.score = self.score + n*125
	self.GameSpeed = self.GameSpeed * 0.95
end

GamePlayer.GetScore = function (self)
	return self.score
end

GamePlayer.GameStart = function (self)
	self.score = 0
	self.game_grid:GameStart()
end

GamePlayer.Pause = function (self,bool)
	if bool then
		self.paused = bool
	else
		self.paused = (self.paused == false)
	end
end

GamePlayer.IsPaused = function (self)
	return self.paused
end

GamePlayer.GameEnd = function (self)
	G_SoundEnd:play()
	SaveHighScore(self.score)
	self.active = false
	self.paused = true
end

GamePlayer.GameStart = function (self)
	self.game_grid:GameStart()
	self.score = 0
	self.GameSpeed = 1
	self.active = true
	self.paused = false
end
